#!/bin/sh

. ../cgi-bin/common/ajax_common

USER_DIR="/usr/ramdisk/etc/webuser"
WEBLOGIN="/usr/ramdisk/tmp/weblogin"
MKCODE="/usr/ramdisk/bin/ipe_mkcode"

WEBACL="${PGETC}/web/webacl.conf"
LOGIN_CONF="${PGETC}/web/login.conf"
PALANGCONF="${PGETC}/palang.conf"
SYSNAMECONF="${PGETC}/sysname.conf"
radis_auth_conf="${PGETC}/web/radius_auth.conf"
ldap_auth_conf="${PGETC}/web/ldap_auth.conf"
panabit_conf="/usr/ramdisk/etc/panabit.inf"

code_file="${WEBLOGIN}/CODE_${REMOTE_ADDR}"
error_file="${WEBLOGIN}/FAIL_${REMOTE_ADDR}"

[ -f ${WEBACL} ] && . ${WEBACL}


load_login_config()
{
	oem="pa"
	SYSNAME=""
	PALANG="ch"

	if [ "${NTM}" = "1" -a "${CMCC}" = "1" ]; then
		oem="cmcc_ntm"
	fi
	
	[ -f ${SYSNAMECONF} ] && . ${SYSNAMECONF}
	[ -f ${LOGIN_CONF}  ] && . ${LOGIN_CONF}
	[ -f ${PALANGCONF} ] && . ${PALANGCONF}
	[ -f ${radis_auth_conf} ] && . ${radis_auth_conf}
	[ -f ${ldap_auth_conf} ] && . ${ldap_auth_conf}
	[ -f ${code_file} ] && verifyCode=1
	[ -f ${panabit_conf} ] && . ${panabit_conf}
	
	[ "${ENONLY}" = "" ] && ENONLY=0
	[ "${NTM}" = "1" ] && CHONLY=1
	blockTime=`verify_errnum get`

	if [ `uname` = "Linux" ]; then
		devid=`${FLOWEYE} key getcookie2 | md5sum | cut -d" " -f1`
	else
		devid=`${FLOWEYE} key getcookie2 | md5`
	fi
	
	sys_type="Ӧ"
	
	if [ "${NTM}" = "1" ]; then
		if [ "${CMCC}" = "1" ]; then
			sys_type="רʿ"
		elif [ "${OEM}" = "pac" ]; then
			sys_type="Ϊϵͳ"
		else
			sys_type="ȫԴϵͳ"
		fi
	fi
	
	[ "${TAP}" = "1" ] && sys_type="TAP"

	[ "${login_title}" != "" ] && sys_type="${login_title}"

	printf "{"
	printf "\"sys_type\":\"${sys_type}\""
	printf ",\"lang\":\"${PALANG}\""
	printf ",\"enonly\":\"${ENONLY}\""
	printf ",\"chonly\":\"${CHONLY}\""
	printf ",\"title\":\"${SYSNAME}\""
	printf ",\"verify_code\":\"${verifyCode}\""
	printf ",\"max_error\":\"${maxErr}\""
	printf ",\"lockTime\":\"${lockTime}\""
	printf ",\"blockTime\":\"${blockTime}\""
	printf ",\"password_ttl\":\"${pswdTTL}\""
	printf ",\"radius_auth\":\"${radius_auth}\""
	printf ",\"ldap_auth\":\"${ldap_auth}\""
	printf ",\"devid\":\"${devid}\""
	printf ",\"oem\":\"${oem}\""
	printf "}"
}


record_errnum()
{
	[ "${maxErr}" = "0" ] && return

	mkdir -p ${WEBLOGIN}
	ERRNUM=1

	if [ -f ${error_file} ]; then
		. ${error_file}
		ERRNUM=$((${ERRNUM}+1))
	fi

	echo "ERRNUM=${ERRNUM}" > ${error_file}
	echo "LEFT=`date +%s`" >> ${error_file}

	if [ ${ERRNUM} -eq ${maxErr} ]; then
		LOG_MSG="¼¼"
		[ "${PALANG}" = "en" ] && LOG_MSG="LOGIN_BLOCK"
		WEB_LOGGER "${src_username}" "${LOG_MSG}" "user=${src_username},ip=${REMOTE_ADDR}"
		retjson 3 "ﵽ¼ҳ"
	fi
}


verify_errnum()
{
	error_file="${WEBLOGIN}/FAIL_${REMOTE_ADDR}"
	
	if [ "${maxErr}" = "0" ]; then
		[ -f "${error_file}" ] && rm ${error_file}
		return
	fi

	[ ! -f "${error_file}" ] && return

	. ${error_file}
	
	[ ${ERRNUM} -lt ${maxErr} ] && return

	now=`date +%s`
	free=$((${now}-${LEFT}))
	if [ ${free} -lt ${lockTime} ]; then
		if [ "${1}" = "get" ]; then
			echo $((${lockTime}-${free}))
		else
			retjson 1 "LOGIN_BLOCK" $((${lockTime}-${free}))
		fi
	else
		rm ${error_file}
	fi
}


cookie_make()
{
	logintime=`date +%s1`
	code=`${MKCODE}`
	esc_code=`${ESCTOOL} -e ${code}`
	username=`${ESCTOOL} -d ${CGI_username}`
	echo "paonline_${username}_$$_${logintime}_${esc_code}|${SERVER_PORT}|"
}


code_make()
{
	mkdir -p ${WEBLOGIN}
	code=`${MKCODE}`
	echo "CODE=${code}" > ${code_file}
	echo "LIVE=`date +%s`" >> ${code_file}
}


code_post()
{	
	[ ! -f ${code_file} ] && code_make

	. ${code_file}
	
	CODE=`${ESCTOOL} -e ${CODE}`
	retjson 0 "OK" "\"${CODE}\""
}


code_cmp()
{
	[ ! -f ${code_file} ] && return
	
	if [ "${CGI_verify_code}" = "" ]; then
		code_make
		retjson 1 "֤"
	fi

	. ${code_file}
	
	now=`date +%s`
	
	if [ $((${now} - ${LIVE})) -gt 60 ]; then
		code_make
		retjson 1 "֤볬ʱ"
	fi

	code_ok=`echo ${CGI_verify_code} | grep -iE "^${CODE}$"`
	if [ "${code_ok}" = "" ]; then
		code_make
		retjson 1 "֤"
	fi

	[ -f ${code_file} ] && rm -rf ${code_file}
}


radius_auth()
{
	RADCLIENT=/usr/ramdisk/bin/radius_client

	. ${PGETC}/ifadmin.conf
	. ${radis_auth_conf}

	cmdargs="${cmdargs} -u ${src_username}"
	cmdargs="${cmdargs} -p ${src_password}"
	cmdargs="${cmdargs} -r ${radius_server}"
	cmdargs="${cmdargs} -i ${ADMIN_IP}"
	cmdargs="${cmdargs} -n ${radius_nas}"
	cmdargs="${cmdargs} -s ${radius_key}"
	cmdargs="${cmdargs} -f ${REMOTE_ADDR}"
	cmdargs="${cmdargs} -t ${radius_timeout}"

	errmsg=`${RADCLIENT} ${cmdargs}`

	if [ $? -ne 0 ]; then
		echo ${errmsg}
	else
		echo "AUTH_OK|0|${radius_level}"
	fi
}


ldap_auth()
{
	LDAPEYE=/usr/ramdisk/bin/ldapeye

	. ${ldap_auth_conf}

	cmdargs="${cmdargs} server=ldap://${ldap_ip}:${ldap_port}"
	cmdargs="${cmdargs} username=${src_username}"
	cmdargs="${cmdargs} password=${src_password}"
	cmdargs="${cmdargs} tree=${ldap_basedn}"

	errmsg=`${LDAPEYE} auth ${cmdargs}`

	if [ $? -ne 0 ]; then
		echo ${errmsg}
	else
		echo "AUTH_OK|0|${ldap_level}"
	fi
}


local_auth()
{
	ufile="${USER_DIR}/${CGI_username}"

	if [ ! -f ${ufile} ]; then
		echo "֤ʧ"
		return 1
	fi

	dir=`dirname ${ufile}`
	
	if [ "${dir}" != ${USER_DIR} ]; then
		echo "֤ʧ"
		return 1
	fi

	. ${ufile}

	[ "${lastedit}" = "" ] && lastedit=`date +%s`
	[ "${pswdTTL}" = "" ] && pswdTTL=0

	# УIPǷ¼
	if [ "${iptab_id}" != "" ]; then
		ipnum=`${FLOWEYE} table get id=${iptab_id} | wc -l | awk '{print $1}'`
		if [ "${ipnum}" != "0" ]; then
			allow_login=`${FLOWEYE} table get allmatched=${REMOTE_ADDR} | grep "${iptab_id} ${iptab_name}"`
			if [ "${allow_login}" = "" ]; then
				echo "IP_DENY"
				return 1
			fi
		fi
	fi

	if [ "${password}" != "${CGI_password}" ]; then
		echo "֤ʧ"
		return 1
	fi

	# Уʱ
	if [ "${pswdTTL}" != "0" ]; then
		_ttl=$((${pswdTTL} - (`date +%s`- ${lastedit})))
		if [  ${_ttl} -le 0 ]; then
			echo "PSWD_TIMEOUT"
			return 1
		else
			pswdLeft=$((${_ttl} / 24 /60 / 60))
		fi
	fi

	if [ "${anylogin}" = "deny" ]; then
		is_login=`ls /usr/ramdisk/weblogin/ | grep "paonline_${name}_"`
		if [ "${is_login}" != "" ]; then
			echo "˺ŵǰѾ¼ҽֹͬʱദ¼"
			return 1
		fi
	fi

	echo "AUTH_OK|${pswdLeft}|${level}"
	return 0
}


user_login()
{
	[ "${CGI_username}" = "" ] && retjson 1 "Чû"
	[ "${CGI_password}" = "" ] && retjson 1 "INV_PASSWORD"
	[ "${CGI_palang}" != "ch" -a "${CGI_palang}" != "en" ] && retjson 1 "INV_LANG"

	export src_username=`${ESCTOOL} -d ${CGI_username}`
	export src_password=`${ESCTOOL} -d ${CGI_password}`

	xss=`echo ${src_username} | grep -E '([<*&/>]|\.\.)'`
	[ "${xss}" != "" ] && retjson 1 "Чû"

	verify_errnum
	code_cmp

	case "${CGI_auth_type_name}" in
		"radius")
			auth_ret=`radius_auth`
			;;

		"ldap")
			auth_ret=`ldap_auth`
			;;

		*)
			auth_ret=`local_auth`
	esac

	login_level=`echo "${auth_ret}" | cut -d"|" -f3`
	pswdLeft=`echo "${auth_ret}" | cut -d"|" -f2`
	auth_ret=`echo "${auth_ret}" | cut -d"|" -f1`

	[ "${pswdLeft}" = "" ] && pswdLeft=0

	# handle auth ret
	case "${auth_ret}" in
		"IP_DENY")
			retjson 4 "IPֹ¼豸"
			;;

		"PSWD_TIMEOUT")
			code_make
			retjson 2 "ʧЧ"
			;;

		"AUTH_OK")
			mkdir -p /usr/ramdisk/weblogin/

			ckinfo=`cookie_make`
			ckfile="/usr/ramdisk/weblogin/${ckinfo}"
			[ "${login_level}" = "" ] && login_level=1

			echo "ip=${REMOTE_ADDR}" > ${ckfile}
			echo "user=${src_username}" >> ${ckfile}
			echo "login_level=${login_level}" >> ${ckfile}
			echo "logintime=`date +%Y.%m.%d/%H:%M:%S`" >> ${ckfile}
			echo "PALANG=${CGI_palang}" > ${PALANGCONF}

			export PANABIT_USER="${src_username}"
			send_syslog "Password auth succeeded for '${src_username}' from ${REMOTE_ADDR}"
			WEB_LOGGER "û¼" "user=${src_username} ip=${REMOTE_ADDR} auth=${CGI_auth_type_name}"
			${FLOWEYE} logger alert "${src_username} login success from ${REMOTE_ADDR}"
			
			[ -f "${error_file}" ] && rm ${error_file}
			[ -f "${code_file}"  ] && rm ${code_file}

			printf "Content-Type:application/json;charset=gb2312\r\n"
			printf "Set-Cookie: ${PANABIT_CKNAME}=${ckinfo};Secure=true;Path=/;HTTPOnly\r\n"
			printf "Set-Cookie: palang=${CGI_palang};Secure=true;Path=/;\r\n"
			printf "\r\n"

			echo -n "{ \"code\": 0, \"msg\": \"Auth_OK\", \"pwleft\": \"${pswdLeft}\"}"
			exit 0
			;;

		*)
			code_make
			record_errnum
			[ "${ERRNUM}" = "" ] && ERRNUM=0
			left_changce=$((${maxErr}-${ERRNUM}))
			[ "${maxErr}" != "0" -a "${maxErr}" != "" ] && auth_ret="${auth_ret}${left_changce}λ"
			send_syslog "Password auth fail for '${src_username}' from ${REMOTE_ADDR}"
			${FLOWEYE} logger alert "${src_username} login FAIL from ${REMOTE_ADDR}"
			WEB_LOGGER  "${src_username}" "û¼ʧ" "user=${src_username},ip=${REMOTE_ADDR}"
			retjson 1 "${auth_ret}"
			;;
	esac
}


edit_password()
{
	[ "${CGI_username}" = "" ] && retjson 1 "NO_USERNAME"

	src_username=`${ESCTOOL} -e ${CGI_username}`
	ufile="${USER_DIR}/${CGI_username}"

	[ ! -f ${ufile} ] && retjson 1 "˺"

	. ${ufile}
	[ "${CGI_cur}" != ${password} ] && retjson 1 "˺"

	echo "name=${name}" > ${ufile}
	echo "password=${CGI_code}" >> ${ufile}
	echo "lastedit=`date +%s`" >> ${ufile}
	echo "level=${level}" >> ${ufile}
	echo "iptab_id=${iptab_id}" >> ${ufile}
	echo "iptab_name=${iptab_name}" >> ${ufile}
	echo "desc=${desc}" >> ${ufile}

	cp "${ufile}" "${PGPATH}/etc/webuser/"
	sync

	WEB_LOGGER "${src_username}" "༭ϵͳû" "username=${src_username}"
	retjson 0 "޸ĳɹ"
}


# action from login.js
case "${CGI_action}" in
	"load_login_config")
		retjson 0 "OK" "`load_login_config`"
		;;

	"getauthcode")
		code_make
		code_post
		;;

	"user_login")
		user_login
		;;

	"edit_password")
		edit_password
		;;
esac


# Login From Cloud or Get userverify.cgi
printf "Content-type: text/html\r\n";
printf "Cache-Control: no-cache\r\n"

if [ "${CGI_pakey}" != "" -a "${CGI_username}" = "admin" ]; then
	keyexist=`${FLOWEYE} cloud list | grep "${CGI_pakey}"`

	if [ "${keyexist}" = "" ]; then
		printf "\r\n"
		echo "<html xmlns:v=\"urn:schemas-microsoft-com:vml\" xmlns:o=\"urn:schemas-microsoft-com:office:office\">"
		echo "<head>"
		echo "<script language=javascript>"
		echo "alert(\"!\");"
		echo "</script>"
		echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=gb2312\">"
		echo "<META HTTP-EQUIV=REFRESH CONTENT=\"0;URL=/login/login.cgi${page_args}\">"
		echo "</head>"
		echo "</html>"

		exit 0
	fi

	for nameval in `${FLOWEYE} key info`
	do
		eval "L_${nameval}"
	done

	paui_vpath=""
	page_args="?license=${L_oem}"

	if [ "${CGI_saaskey}" != "" ]; then
		paui_vpath="/VPATH_${CGI_saaskey}"
		page_args="${page_args}&saaskey=${CGI_saaskey}"
	fi

	mkdir -p /usr/ramdisk/weblogin/
	ckinfo=`cookie_make`
	ckfile="/usr/ramdisk/weblogin/${ckinfo}"

	${FLOWEYE} cloud remove key=${CGI_pakey}

	[ -f ${PALANGCONF} ] && . ${PALANGCONF}
	[ "${PALANG}" = "" ] && PALANG="ch"
	
	echo "ip=${REMOTE_ADDR}" > ${ckfile}
	echo "user=${CGI_username}" >> ${ckfile}
	echo "login_level=0" >> ${ckfile}
	echo "logintime=`date +%Y.%m.%d/%H:%M:%S`" >> ${ckfile}

	printf "Set-Cookie: ${PANABIT_CKNAME}=${ckinfo};Secure=true;Path=/;HTTPOnly;\r\n"
	printf "Set-Cookie: palang=${PALANG};Secure=true;Path=/;\r\n"		
	printf "\r\n"

	echo "<html xmlns:v=\"urn:schemas-microsoft-com:vml\" xmlns:o=\"urn:schemas-microsoft-com:office:office\">"
	echo "<head>"
	echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=gb2312\">"
	echo "<META HTTP-EQUIV=REFRESH CONTENT=\"0;URL=${paui_vpath}/${page_args}\">"
	echo "</head>"
	echo "</html>"

	send_syslog "Password auth succeeded for 'admin@cloud' from ${REMOTE_ADDR}"
	WEB_LOGGER  "admin" "ûƽ̨¼" "user=admin@cloud,ip=${REMOTE_ADDR}"
	${FLOWEYE} logger alert "admin@cloud login success from ${REMOTE_ADDR}"

	exit 0
fi

printf "\r\n"
echo "<html xmlns:v=\"urn:schemas-microsoft-com:vml\" xmlns:o=\"urn:schemas-microsoft-com:office:office\">"
echo "<head>"
echo "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=gb2312\">"
echo "<meta HTTP-EQUIV=REFRESH CONTENT=\"0;URL=/login/login.cgi\">"
echo "</head>"
echo "</html>"
